@@ -10,10 +10,13 @@ import (
 	"strings"
 
 	"github.com/Masterminds/semver"
+	"github.com/adrg/xdg"
 	"github.com/mitchellh/go-homedir"
 	"gopkg.in/yaml.v3"
 )
 
+const ApplicationName = "octosql"
+
 var octosqlHomeDir = func() string {
 	dir, err := homedir.Dir()
 	if err != nil {
@@ -22,6 +25,30 @@ var octosqlHomeDir = func() string {
 	return filepath.Join(dir, ".octosql")
 }()
 
+var OctosqlConfigDir = func() string {
+	configDirPath, err := xdg.SearchConfigFile(ApplicationName)
+	if err != nil {
+		return octosqlHomeDir
+	}
+	return configDirPath
+}()
+
+var OctosqlCacheDir = func() string {
+	cacheDirPath, err := xdg.SearchCacheFile(ApplicationName)
+	if err != nil {
+		return octosqlHomeDir
+	}
+	return cacheDirPath
+}()
+
+var OctosqlDataDir = func() string {
+	dataDirPath, err := xdg.SearchDataFile(ApplicationName)
+	if err != nil {
+		return octosqlHomeDir
+	}
+	return dataDirPath
+}()
+
 type Config struct {
 	Databases []DatabaseConfig `yaml:"databases"`
 	Files     FilesConfig      `yaml:"files"`
@@ -45,7 +72,7 @@ type JSONConfig struct {
 
 func Read() (*Config, error) {
 	var config Config
-	data, err := ioutil.ReadFile(filepath.Join(octosqlHomeDir, "octosql.yml"))
+	data, err := ioutil.ReadFile(filepath.Join(OctosqlConfigDir, "octosql.yml"))
 	if err != nil && os.IsNotExist(err) {
 		config = Config{}
 	} else if err != nil {
